@@ -39,6 +39,8 @@
 #include "smalloc.h"
 #include "verify.h"
 #include "diskutil.h"
+#include "cgroup.h"
+#include "profile.h"
 
 unsigned long page_mask;
 unsigned long page_size;
@@ -60,6 +62,8 @@ static volatile int fio_abort;
 static int exit_value;
 static struct itimerval itimer;
 static pthread_t gtod_thread;
+static struct flist_head *cgroup_list;
+static char *cgroup_mnt;
 
 struct io_log *agg_io_log[2];
 
@@ -131,7 +135,7 @@ static void sig_quit(int sig)
 static void sig_int(int sig)
 {
 	if (threads) {
-		printf("\nfio: terminating on signal %d\n", sig);
+		log_info("\nfio: terminating on signal %d\n", sig);
 		fflush(stdout);
 		terminate_threads(TERMINATE_ALL);
 	}
@@ -429,6 +433,8 @@ static void do_verify(struct thread_data *td)
 	int ret, min_events;
 	unsigned int i;
 
+	dprint(FD_VERIFY, "starting loop\n");
+
 	/*
 	 * sync io first and invalidate cache, to make sure we really
 	 * read from disk.
@@ -536,7 +542,8 @@ sync_done:
 		 */
 		full = queue_full(td) || ret == FIO_Q_BUSY;
 		if (full || !td->o.iodepth_batch_complete) {
-			min_events = td->o.iodepth_batch_complete;
+			min_events = min(td->o.iodepth_batch_complete,
+					 td->cur_depth);
 			if (full && !min_events)
 				min_events = 1;
 
@@ -565,6 +572,8 @@ sync_done:
 		cleanup_pending_aio(td);
 
 	td_set_runstate(td, TD_RUNNING);
+
+	dprint(FD_VERIFY, "exiting loop\n");
 }
 
 /*
@@ -581,7 +590,8 @@ static void do_io(struct thread_data *td)
 	else
 		td_set_runstate(td, TD_RUNNING);
 
-	while ((td->this_io_bytes[0] + td->this_io_bytes[1]) < td->o.size) {
+	while ( (td->o.read_iolog_file && !flist_empty(&td->io_log_list)) ||
+	        ((td->this_io_bytes[0] + td->this_io_bytes[1]) < td->o.size) ) {
 		struct timeval comp_time;
 		unsigned long bytes_done[2] = { 0, 0 };
 		int min_evts = 0;
@@ -687,7 +697,8 @@ sync_done:
 		 */
 		full = queue_full(td) || ret == FIO_Q_BUSY;
 		if (full || !td->o.iodepth_batch_complete) {
-			min_evts = td->o.iodepth_batch_complete;
+			min_evts = min(td->o.iodepth_batch_complete,
+					td->cur_depth);
 			if (full && !min_evts)
 				min_evts = 1;
 
@@ -1035,6 +1046,15 @@ static void *thread_main(void *data)
 	 */
 	fio_mutex_remove(td->mutex);
 
+	if (td->o.uid != -1U && setuid(td->o.uid)) {
+		td_verror(td, errno, "setuid");
+		goto err;
+	}
+	if (td->o.gid != -1U && setgid(td->o.gid)) {
+		td_verror(td, errno, "setgid");
+		goto err;
+	}
+
 	/*
 	 * May alter parameters that init_io_u() will use, so we need to
 	 * do this first.
@@ -1072,6 +1092,9 @@ static void *thread_main(void *data)
 		}
 	}
 
+	if (td->o.cgroup_weight && cgroup_setup(td, cgroup_list, &cgroup_mnt))
+		goto err;
+
 	if (nice(td->o.nice) == -1) {
 		td_verror(td, errno, "nice");
 		goto err;
@@ -1192,11 +1215,16 @@ static void *thread_main(void *data)
 
 err:
 	if (td->error)
-		printf("fio: pid=%d, err=%d/%s\n", (int) td->pid, td->error,
+		log_info("fio: pid=%d, err=%d/%s\n", (int) td->pid, td->error,
 							td->verror);
+
+	if (td->o.verify_async)
+		verify_async_exit(td);
+
 	close_and_free_files(td);
 	close_ioengine(td);
 	cleanup_io_u(td);
+	cgroup_shutdown(td, &cgroup_mnt);
 
 	if (td->o.cpumask_set) {
 		int ret = fio_cpuset_exit(&td->o.cpumask);
@@ -1204,9 +1232,6 @@ err:
 		td_verror(td, ret, "fio_cpuset_exit");
 	}
 
-	if (td->o.verify_async)
-		verify_async_exit(td);
-
 	/*
 	 * do this very late, it will log file closing as well
 	 */
@@ -1392,17 +1417,17 @@ static void run_threads(void)
 		return;
 
 	if (!terse_output) {
-		printf("Starting ");
+		log_info("Starting ");
 		if (nr_thread)
-			printf("%d thread%s", nr_thread,
+			log_info("%d thread%s", nr_thread,
 						nr_thread > 1 ? "s" : "");
 		if (nr_process) {
 			if (nr_thread)
 				printf(" and ");
-			printf("%d process%s", nr_process,
+			log_info("%d process%s", nr_process,
 						nr_process > 1 ? "es" : "");
 		}
-		printf("\n");
+		log_info("\n");
 		fflush(stdout);
 	}
 
@@ -1633,6 +1658,9 @@ int main(int argc, char *argv[])
 	if (parse_options(argc, argv))
 		return 1;
 
+	if (exec_profile && load_profile(exec_profile))
+		return 1;
+
 	if (!thread_number)
 		return 0;
 
@@ -1648,6 +1676,9 @@ int main(int argc, char *argv[])
 
 	status_timer_arm();
 
+	cgroup_list = smalloc(sizeof(*cgroup_list));
+	INIT_FLIST_HEAD(cgroup_list);
+
 	run_threads();
 
 	if (!fio_abort) {
@@ -1659,6 +1690,11 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	cgroup_kill(cgroup_list);
+	sfree(cgroup_list);
+	if (cgroup_mnt)
+		sfree(cgroup_mnt);
+
 	fio_mutex_remove(startup_mutex);
 	fio_mutex_remove(writeout_mutex);
 	return exit_value;
