@@ -28,6 +28,9 @@
 #include "io_ddir.h"
 #include "ioengine.h"
 #include "iolog.h"
+#include "helpers.h"
+#include "options.h"
+#include "profile.h"
 
 #ifdef FIO_HAVE_GUASI
 #include <guasi.h>
@@ -63,6 +66,8 @@ enum fio_memtype {
 #define FIO_IO_U_LAT_U_NR 10
 #define FIO_IO_U_LAT_M_NR 12
 
+#define MAX_PATTERN_SIZE 512
+
 struct thread_stat {
 	char *name;
 	char *verror;
@@ -175,7 +180,7 @@ struct thread_options {
 	unsigned int verifysort;
 	unsigned int verify_interval;
 	unsigned int verify_offset;
-	unsigned int verify_pattern;
+	char verify_pattern[MAX_PATTERN_SIZE];
 	unsigned int verify_pattern_bytes;
 	unsigned int verify_fatal;
 	unsigned int verify_async;
@@ -223,6 +228,7 @@ struct thread_options {
 	unsigned int file_service_type;
 	unsigned int group_reporting;
 	unsigned int fadvise_hint;
+	unsigned int fallocate;
 	unsigned int zero_buffers;
 	unsigned int refill_buffers;
 	unsigned int time_based;
@@ -262,6 +268,22 @@ struct thread_options {
 	 * I/O Error handling
 	 */
 	unsigned int continue_on_error;
+
+	/*
+	 * Benchmark profile type
+	 */
+	char *profile;
+
+	/*
+	 * blkio cgroup support
+	 */
+	char *cgroup;
+	unsigned int cgroup_weight;
+
+	unsigned int uid;
+	unsigned int gid;
+
+	unsigned int sync_file_range;
 };
 
 #define FIO_VERROR_SIZE	128
@@ -341,7 +363,7 @@ struct thread_data {
 	/*
 	 * Rate state
 	 */
-	unsigned long rate_usec_cycle[2];
+	unsigned long rate_nsec_cycle[2];
 	long rate_pending_usleep[2];
 	unsigned long rate_bytes[2];
 	unsigned long rate_blocks[2];
@@ -397,6 +419,8 @@ struct thread_data {
 	unsigned int file_service_left;
 	struct fio_file *file_service_file;
 
+	unsigned int sync_file_range_nr;
+
 	/*
 	 * For generating file sizes
 	 */
@@ -407,6 +431,12 @@ struct thread_data {
 	 */
 	unsigned int total_err_count;
 	int first_error;
+
+	/*
+	 * Can be overloaded by profiles
+	 */
+	struct prof_io_ops prof_io_ops;
+	void *prof_data;
 };
 
 /*
@@ -519,8 +549,9 @@ extern void fio_options_dup_and_init(struct option *);
 extern void options_mem_dupe(struct thread_data *);
 extern void options_mem_free(struct thread_data *);
 extern void td_fill_rand_seeds(struct thread_data *);
+extern void add_job_opts(const char **);
 #define FIO_GETOPT_JOB		0x89988998
-#define FIO_NR_OPTIONS		128
+#define FIO_NR_OPTIONS		(FIO_MAX_OPTS + 128)
 
 /*
  * ETA/status stuff
