@@ -1,6 +1,8 @@
 #ifndef FIO_LOG_H
 #define FIO_LOG_H
 
+#include <stdio.h>
+
 extern FILE *f_out;
 extern FILE *f_err;
 
@@ -14,6 +16,7 @@ extern FILE *f_err;
 	} while (0)
 
 #define log_info(args...)	fprintf(f_out, ##args)
+#define log_valist(str, args)	vfprintf(f_out, (str), (args))
 
 FILE *get_f_out(void);
 FILE *get_f_err(void);
