@@ -2,6 +2,7 @@
 #define FIO_OS_SOLARIS_H
 
 #include <errno.h>
+#include <malloc.h>
 #include <sys/types.h>
 #include <sys/fcntl.h>
 #include <sys/pset.h>
@@ -12,6 +13,8 @@
 #define FIO_HAVE_POSIXAIO_FSYNC
 #define FIO_HAVE_CPU_AFFINITY
 #define FIO_HAVE_PSHARED_MUTEX
+#define FIO_USE_GENERIC_BDEV_SIZE
+#define FIO_HAVE_FDATASYNC
 
 #define OS_MAP_ANON		MAP_ANON
 #define OS_RAND_MAX		2147483648UL
@@ -23,17 +26,6 @@ struct solaris_rand_seed {
 typedef psetid_t os_cpu_mask_t;
 typedef struct solaris_rand_seed os_random_state_t;
 
-static inline int blockdev_size(int fd, unsigned long long *bytes)
-{
-	off_t end = lseek(fd, 0, SEEK_END);
-
-	if (end < 0)
-		return errno;
-
-	*bytes = end;
-	return 0;
-}
-
 static inline int blockdev_invalidate_cache(int fd)
 {
 	return EINVAL;
